﻿using System;
using System.Linq;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

using Pong.Behaviours;

using Syntriax.Engine.Core;
using Syntriax.Engine.Core.Factory;
using Syntriax.Engine.Network;
using Syntriax.Engine.Physics2D;
using Syntriax.Engine.Systems.Tween;
using Syntriax.Engine.Integration.MonoGame;

namespace Pong;

public class GamePong : MonoGameWindow
{
    private readonly IUniverseObject platformSpecificUniverseObject = null!;

    public GamePong(IUniverseObject platformSpecificUniverseObject)
    {
        this.platformSpecificUniverseObject = platformSpecificUniverseObject;
        Graphics.PreferredBackBufferWidth = 1024;
        Graphics.PreferredBackBufferHeight = 576;
        Graphics.GraphicsProfile = GraphicsProfile.HiDef;
    }

    protected override void PopulateUniverse(IUniverse universe)
    {
        SpriteFont spriteFont = Content.Load<SpriteFont>("UbuntuMono");

        universe.Register(platformSpecificUniverseObject);

        if (Environment.GetCommandLineArgs().FirstOrDefault(x => x.CompareTo("-server") == 0) is not null)
        {
            LiteNetLibServer server = universe.InstantiateUniverseObject<LiteNetLibServer>().SetUniverseObject("Server");
            UniverseObjectFactory.Instantiate<NetworkManager>().SetUniverseObject("NetworkManager", server.BehaviourController.UniverseObject);
            server.Start(8888, 2);
            Window.Title = $"Server";
        }
        else
        {
            LiteNetLibClient client = universe.InstantiateUniverseObject<LiteNetLibClient>().SetUniverseObject("Client");
            UniverseObjectFactory.Instantiate<NetworkManager>().SetUniverseObject("NetworkManager", client.BehaviourController.UniverseObject);
            client.Connect("localhost", 8888);
            Window.Title = $"Client";

            DrawManager drawManager = universe.InstantiateUniverseObject<DrawManager>().SetUniverseObject("Draw Manager");
            universe.InstantiateUniverseObject().SetUniverseObject("Shape Batcher", drawManager).BehaviourController.AddBehaviour<ShapeBatcher>();
            universe.InstantiateUniverseObject().SetUniverseObject("Sprite Batcher", drawManager).BehaviourController.AddBehaviour<SpriteBatcher>();
        }

        universe.InstantiateUniverseObject<UpdateManager>().SetUniverseObject("Update Manager");
        universe.InstantiateUniverseObject<CoroutineManager>().SetUniverseObject("Coroutine Manager");
        universe.InstantiateUniverseObject<TweenManager>().SetUniverseObject("Tween Manager");
        universe.InstantiateUniverseObject<PhysicsEngine2D>().SetUniverseObject("Physics Engine 2D");

        ////////////////////////////////////////////////////////////////////////////////////

        universe.InstantiateUniverseObject().SetUniverseObject("Camera")
            .BehaviourController.AddBehaviour<Transform2D>()
            .BehaviourController.AddBehaviour<CameraController>()
            .BehaviourController.AddBehaviour<MonoGameCamera2DBehaviour>();

        ////////////////////////////////////////////////////////////////////////////////////

        PongManagerBehaviour pongManager = universe.InstantiateUniverseObject().SetUniverseObject("Pong Game Manager")
            .BehaviourController.AddBehaviour<PongManagerBehaviour>(5);

        ////////////////////////////////////////////////////////////////////////////////////

        universe.InstantiateUniverseObject().SetUniverseObject("Ball")
            .BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(0, 0f), scale: new Vector2D(10f, 10f))
            .BehaviourController.AddBehaviour<CircleBehaviour>(new Circle(Vector2D.Zero, 1f))
            .BehaviourController.AddBehaviour<BallBehaviour>()
            .BehaviourController.AddBehaviour<RigidBody2D>();

        ////////////////////////////////////////////////////////////////////////////////////

        IUniverseObject leftPaddle = UniverseObjectFactory.Instantiate().SetUniverseObject("Left Paddle");
        leftPaddle.BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(-468f, 0f), scale: new Vector2D(15f, 60f))
            .BehaviourController.AddBehaviour<ShapeBehaviour>(Shape2D.Square)
            .BehaviourController.AddBehaviour<RigidBody2D>();
        PaddleBehaviour leftPaddleBehaviour = BehaviourFactory.Instantiate<PaddleBehaviour>(Keys.W, Keys.S, 228f, -228f, 400f);
        leftPaddleBehaviour.Id = "lp";
        leftPaddle.BehaviourController.AddBehaviour(leftPaddleBehaviour);
        universe.Register(leftPaddle);

        IUniverseObject rightPaddle = UniverseObjectFactory.Instantiate().SetUniverseObject("Right Paddle");
        rightPaddle.BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(468f, 0f), scale: new Vector2D(15f, 60f))
            .BehaviourController.AddBehaviour<ShapeBehaviour>(Shape2D.Square)
            .BehaviourController.AddBehaviour<RigidBody2D>();
        PaddleBehaviour rightPaddleBehaviour = BehaviourFactory.Instantiate<PaddleBehaviour>(Keys.Up, Keys.Down, 228f, -228f, 400f);
        rightPaddleBehaviour.Id = "rp";
        rightPaddle.BehaviourController.AddBehaviour(rightPaddleBehaviour);
        universe.Register(rightPaddle);

        ////////////////////////////////////////////////////////////////////////////////////

        universe.InstantiateUniverseObject().SetUniverseObject("Wall Top")
            .BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(0f, 308f), scale: new Vector2D(552f, 20f))
            .BehaviourController.AddBehaviour<ShapeBehaviour>(Shape2D.Square)
            .BehaviourController.AddBehaviour<RigidBody2D>().IsStatic = true;

        universe.InstantiateUniverseObject().SetUniverseObject("Wall Bottom")
            .BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(0f, -308f), scale: new Vector2D(552f, 20f))
            .BehaviourController.AddBehaviour<ShapeBehaviour>(Shape2D.Square)
            .BehaviourController.AddBehaviour<RigidBody2D>().IsStatic = true;

        universe.InstantiateUniverseObject().SetUniverseObject("Wall Right")
            .BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(532f, 0f), scale: new Vector2D(20f, 328f))
            .BehaviourController.AddBehaviour<WallScoreBehaviour>((Action)pongManager.ScoreToLeft)
            .BehaviourController.AddBehaviour<ShapeBehaviour>(Shape2D.Square)
            .BehaviourController.AddBehaviour<RigidBody2D>().IsStatic = true;

        universe.InstantiateUniverseObject().SetUniverseObject("Wall Left")
            .BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(-532f, 0f), scale: new Vector2D(20f, 328f))
            .BehaviourController.AddBehaviour<WallScoreBehaviour>((Action)pongManager.ScoreToRight)
            .BehaviourController.AddBehaviour<ShapeBehaviour>(Shape2D.Square)
            .BehaviourController.AddBehaviour<RigidBody2D>().IsStatic = true;

        ////////////////////////////////////////////////////////////////////////////////////

        universe.InstantiateUniverseObject().SetUniverseObject("Score Left")
            .BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(-250f, 250f), scale: Vector2D.One * .25f)
            .BehaviourController.AddBehaviour<TextScoreBehaviour>(true, spriteFont);

        universe.InstantiateUniverseObject().SetUniverseObject("Score Right")
            .BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(250f, 250f), scale: Vector2D.One * .25f)
            .BehaviourController.AddBehaviour<TextScoreBehaviour>(false, spriteFont);
    }
}
